package cypher

import (
	"encoding/json"
	"fmt"
	"strings"
	"testing"

	"git.science.uu.nl/graphpolaris/query-conversion/entity"
	"github.com/stretchr/testify/assert"
)

// All these tests test the entire flow

func Test1(t *testing.T) {
	// Works, but, the AVG function is applied to a string, so that doesnt work, but the translation does :D
	query := []byte(`{
		"databaseName": "Movies3",
		"return": {
			"entities": [
				0,
				1,
				2
			],
			"relations": [
				0,
				1
			],
			"groupBys": [
				0
			]
		},
		"entities": [
			{
				"id": 0,
				"name": "Person",
				"constraints": [
				{
					"attribute": "name",
					"value": "Raymond Campbell",
					"dataType": "string",
					"matchType": "NEQ",
					"inID": -1,
					"inType": ""
				}
				]
			},
			{
				"id": 1,
				"name": "Movie",
				"constraints": []
			},
			{
				"id": 2,
				"name": "Genre",
				"constraints": []
			}
		],
		"relations": [
			{
				"id": 0,
				"name": "DIRECTED",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 0,
				"toType": "entity",
				"toID": 1,
				"constraints": []
			},
			{
				"id": 1,
				"name": "IN_GENRE",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "groupBy",
				"fromID": 0,
				"toType": "entity",
				"toID": 2,
				"constraints": []
			}
		],
		"groupBys": [
			{
				"id": 0,
				"groupType": "entity",
				"groupID": 0,
				"groupAttribute": "bornIn",
				"byType": "entity",
				"byID": 1,
				"byAttribute": "imdbId",
				"appliedModifier": "AVG",
				"relationID": 0,
				"constraints": []
			}
		],
		"machineLearning": [],
		"limit": 5000
	}`)

	var JSONQuery entity.IncomingQueryJSON
	json.Unmarshal(query, &JSONQuery)

	s := NewService()
	cypher, err := s.ConvertQuery(&JSONQuery)
	if err != nil {
		fmt.Println(err)
	}

	answer := `MATCH p0 = (e0:Person)-[:DIRECTED*1..1]-(e1:Movie)
		WHERE  e0.name <> "Raymond Campbell" 
	UNWIND relationships(p0) as r0 
	WITH *
	WITH e1.imdbId AS e1_imdbId, AVG(e0.bornIn) AS AVG_bornIn 
	MATCH p1 = (eg1:Movie)-[:IN_GENRE*1..1]-(e2:Genre)
		WHERE eg1.imdbId IN e1_imdbId 
	UNWIND relationships(p1) as r1 
	WITH *
	RETURN  r1, eg1, e2
	LIMIT 5000`

	fmt.Println(*cypher)
	trimmedCypher := strings.Replace(*cypher, "\n", "", -1)
	trimmedCypher = strings.Replace(trimmedCypher, "\t", "", -1)

	trimmedAnswer := strings.Replace(answer, "\n", "", -1)
	trimmedAnswer = strings.Replace(trimmedAnswer, "\t", "", -1)

	fmt.Println(*cypher)
	assert.Equal(t, trimmedAnswer, trimmedAnswer)

}
func Test2(t *testing.T) {
	// Works, but, the AVG function is applied to a string, so that doesnt work, but the translation does :D
	query := []byte(`{
		"databaseName": "TweedeKamer",
		"return": {
			"entities": [
				0,
				1,
				2
			],
			"relations": [
				0,
				1
			]
		},
		"entities": [
			{
				"name": "parliament",
				"ID": 0,
				"constraints": [
					{
						"attribute": "name",
						"value": "Geert",
						"dataType": "string",
						"matchType": "contains",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "parties",
				"ID": 1,
				"constraints": []
			},
			{
				"name": "resolutions",
				"ID": 2,
				"constraints": []
			}
		],
		"relations": [
			{
				"ID": 0,
				"name": "member_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 0,
				"toType": "entity",
				"toID": 1,
				"constraints": []
			},
			{
				"ID": 1,
				"name": "submits",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 0,
				"toType": "entity",
				"toID": 2,
				"constraints": []
			}
		],
		"groupBys": [],
		"machineLearning": [],
		"limit": 5000
	}
	`)

	var JSONQuery entity.IncomingQueryJSON
	json.Unmarshal(query, &JSONQuery)

	s := NewService()
	cypher, err := s.ConvertQuery(&JSONQuery)
	if err != nil {
		fmt.Println(err)
	}

	answer := `MATCH p0 = (e0:parliament)-[:member_of*1..1]-(e1:parties)
		WHERE  e0.name CONTAINS "%Geert%" 
	UNWIND relationships(p0) as r0 
	WITH *
	MATCH p1 = (e0:parliament)-[:submits*1..1]-(e2:resolutions)
		WHERE  e0.name CONTAINS "%Geert%" 
	UNWIND relationships(p1) as r1 
	WITH *
	RETURN  r1, e0, e2, r0, e0, e1
	LIMIT 5000`

	fmt.Println(*cypher)

	trimmedCypher := strings.Replace(*cypher, "\n", "", -1)
	trimmedCypher = strings.Replace(trimmedCypher, "\t", "", -1)

	trimmedAnswer := strings.Replace(answer, "\n", "", -1)
	trimmedAnswer = strings.Replace(trimmedAnswer, "\t", "", -1)

	fmt.Println(*cypher)
	assert.Equal(t, trimmedAnswer, trimmedAnswer)

}
func Test3(t *testing.T) {
	// Works, but, the AVG function is applied to a string, so that doesnt work, but the translation does :D
	query := []byte(`{
		"databaseName": "TweedeKamer",
		"return": {
			"entities": [
				0,
				1,
				2,
				3,
				4
			],
			"relations": [
				0,
				1,
				2,
				3
			]
		},
		"entities": [
			{
				"name": "parliament",
				"ID": 0,
				"constraints": [
					{
						"attribute": "name",
						"value": "A",
						"dataType": "string",
						"matchType": "contains",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "parties",
				"ID": 1,
				"constraints": [
					{
						"attribute": "seats",
						"value": "10",
						"dataType": "int",
						"matchType": "LT",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "resolutions",
				"ID": 2,
				"constraints": [
					{
						"attribute": "date",
						"value": "mei",
						"dataType": "string",
						"matchType": "contains",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "parliament",
				"ID": 3,
				"constraints": []
			},
			{
				"name": "parties",
				"ID": 4,
				"constraints": [
					{
						"attribute": "name",
						"value": "Volkspartij voor Vrijheid en Democratie",
						"dataType": "string",
						"matchType": "==",
						"inID": -1,
						"inType": ""
					}
				]
			}
		],
		"relations": [
			{
				"ID": 0,
				"name": "member_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 0,
				"toType": "entity",
				"toID": 1,
				"constraints": []
			},
			{
				"ID": 1,
				"name": "submits",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 0,
				"toType": "entity",
				"toID": 2,
				"constraints": []
			},
			{
				"ID": 2,
				"name": "submits",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 3,
				"toType": "entity",
				"toID": 2,
				"constraints": []
			},
			{
				"ID": 3,
				"name": "member_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 3,
				"toType": "entity",
				"toID": 4,
				"constraints": []
			}
		],
		"groupBys": [],
		"machineLearning": [],
		"limit": 5000
	}
	
	`)

	var JSONQuery entity.IncomingQueryJSON
	json.Unmarshal(query, &JSONQuery)

	s := NewService()
	cypher, err := s.ConvertQuery(&JSONQuery)
	if err != nil {
		fmt.Println(err)
	}

	answer := `MATCH p0 = (e0:parliament)-[:member_of*1..1]-(e1:parties)
		WHERE  e0.name CONTAINS "%A%" 
		AND  e1.seats < 10 
	UNWIND relationships(p0) as r0 
	WITH *
	MATCH p1 = (e0:parliament)-[:submits*1..1]-(e2:resolutions)
		WHERE  e0.name CONTAINS "%A%" 
		AND  e2.date CONTAINS "%mei%" 
	UNWIND relationships(p1) as r1 
	WITH *
	MATCH p2 = (e3:parliament)-[:submits*1..1]-(e2:resolutions)
		WHERE  e2.date CONTAINS "%mei%" 
	UNWIND relationships(p2) as r2 
	WITH *
	MATCH p3 = (e3:parliament)-[:member_of*1..1]-(e4:parties)
		WHERE  e4.name = "Volkspartij voor Vrijheid en Democratie" 
	UNWIND relationships(p3) as r3 
	WITH *
	RETURN  r3, e3, e4, r2, e3, e2, r1, e0, e2, r0, e0, e1
	LIMIT 5000`

	fmt.Println(*cypher)

	trimmedCypher := strings.Replace(*cypher, "\n", "", -1)
	trimmedCypher = strings.Replace(trimmedCypher, "\t", "", -1)

	trimmedAnswer := strings.Replace(answer, "\n", "", -1)
	trimmedAnswer = strings.Replace(trimmedAnswer, "\t", "", -1)

	fmt.Println(*cypher)
	assert.Equal(t, trimmedAnswer, trimmedAnswer)

}

func Test4(t *testing.T) {
	// Works, but, the AVG function is applied to a string, so that doesnt work, but the translation does :D
	query := []byte(`{
		"entities": [
			{
				"name": "parliament",
				"ID": 0,
				"constraints": [
					{
						"attribute": "name",
						"value": "Geert",
						"dataType": "string",
						"matchType": "contains",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "commissions",
				"ID": 1,
				"constraints": []
			},
			{
				"name": "parliament",
				"ID": 2,
				"constraints": []
			},
			{
				"name": "parties",
				"ID": 3,
				"constraints": [
					{
						"attribute": "seats",
						"value": "10",
						"dataType": "int",
						"matchType": "LT",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "resolutions",
				"ID": 4,
				"constraints": [
					{
						"attribute": "date",
						"value": "mei",
						"dataType": "string",
						"matchType": "contains",
						"inID": -1,
						"inType": ""
					}
				]
			},
			{
				"name": "resolutions",
				"ID": 5,
				"constraints": []
			},
			{
				"name": "parties",
				"ID": 6,
				"constraints": []
			}
			,
			{
				"name": "parliament",
				"ID": 7,
				"constraints": []
			}
		   
		],
		"groupBys": [],
		"relations": [
			{
				"ID": 0,
				"name": "part_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 0,
				"toType": "entity",
				"toID": 1,
				"constraints": []
			},
			{
				"ID": 1,
				"name": "part_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 2,
				"toType": "entity",
				"toID": 1,
				"constraints": []
			},
			{
				"ID": 2,
				"name": "member_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 2,
				"toType": "entity",
				"toID": 3,
				"constraints": []
			},
			{
				"ID": 3,
				"name": "submits",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 2,
				"toType": "entity",
				"toID": 4,
				"constraints": []
			},
			{
				"ID": 4,
				"name": "submits",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 0,
				"toType": "entity",
				"toID": 5,
				"constraints": []
			},
			{
				"ID": 5,
				"name": "member_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 0,
				"toType": "entity",
				"toID": 6,
				"constraints": []
			}
			,
			{
				"ID": 6,
				"name": "member_of",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromId": 7,
				"toType": "entity",
				"toID": 6,
				"constraints": []
			}
		],
		"machineLearning": [],
		"limit": 5000
	}
	`)

	var JSONQuery entity.IncomingQueryJSON
	json.Unmarshal(query, &JSONQuery)

	s := NewService()
	cypher, err := s.ConvertQuery(&JSONQuery)
	if err != nil {
		fmt.Println(err)
	}

	answer := `MATCH p2 = (e2:parliament)-[:member_of*1..1]-(e3:parties)
		WHERE  e3.seats < 10 
	UNWIND relationships(p2) as r2 
	WITH *
	MATCH p3 = (e2:parliament)-[:submits*1..1]-(e4:resolutions)
		WHERE  e4.date CONTAINS "%mei%" 
	UNWIND relationships(p3) as r3 
	WITH *
	MATCH p6 = (e7:parliament)-[:member_of*1..1]-(e6:parties)
	UNWIND relationships(p6) as r6 
	WITH *
	MATCH p4 = (e0:parliament)-[:submits*1..1]-(e5:resolutions)
		WHERE  e0.name CONTAINS "%%Geert%" 
	UNWIND relationships(p4) as r4 
	WITH *
	MATCH p0 = (e0:parliament)-[:part_of*1..1]-(e1:commissions)
		WHERE  e0.name CONTAINS "%%Geert%" 
	UNWIND relationships(p0) as r0 
	WITH *
	MATCH p5 = (e0:parliament)-[:member_of*1..1]-(e6:parties)
		WHERE  e0.name CONTAINS "%%Geert%" 
	UNWIND relationships(p5) as r5 
	WITH *
	MATCH p1 = (e2:parliament)-[:part_of*1..1]-(e1:commissions)
	UNWIND relationships(p1) as r1 
	WITH *
	RETURN  r1, e2, e1, r5, e0, e6, r0, e0, e1, r4, e0, e5, r6, e7, e6, r3, e2, e4, r2, e2, e3
	LIMIT 5000`

	fmt.Println(*cypher)
	trimmedCypher := strings.Replace(*cypher, "\n", "", -1)
	trimmedCypher = strings.Replace(trimmedCypher, "\t", "", -1)

	trimmedAnswer := strings.Replace(answer, "\n", "", -1)
	trimmedAnswer = strings.Replace(trimmedAnswer, "\t", "", -1)

	fmt.Println(*cypher)
	assert.Equal(t, trimmedAnswer, trimmedAnswer)

}
func Test5(t *testing.T) {
	// Works, but, the AVG function is applied to a string, so that doesnt work, but the translation does :D
	query := []byte(`{
		"databaseName": "Movies3",
		"entities": [
			{
				"id": 0,
				"name": "Person",
				"constraints": []
			},
			{
				"id": 1,
				"name": "Movie",
				"constraints": []
			},
			{
				"id": 2,
				"name": "Person",
				"constraints": [
					{
						"attribute": "bornIn",
						"value": "",
						"dataType": "string",
						"matchType": "",
						"inID": 0,
						"inType": "groupBy"
					}
				]
			}
		],
		"relations": [
			{
				"id": 0,
				"name": "ACTED_IN",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 0,
				"toType": "entity",
				"toID": 1,
				"constraints": []
			}
		],
		"groupBys": [
			{
				"id": 0,
				"groupType": "entity",
				"groupID": 1,
				"groupAttribute": "imdbRating",
				"byType": "entity",
				"byID": 0,
				"byAttribute": "bornIn",
				"appliedModifier": "AVG",
				"relationID": 0,
				"constraints": [
					{
						"attribute": "imdbRating",
						"value": "7.5",
						"dataType": "int",
						"matchType": "GT",
						"inID": -1,
						"inType": ""
					}
				]
			}
		],
		"machineLearning": [],
		"limit": 5000
	}
	`)

	answer := `MATCH p0 = (e0:Person)-[:ACTED_IN*1..1]-(e1:Movie)
	UNWIND relationships(p0) as r0 
	WITH *
	WITH e0.bornIn AS e0_bornIn, AVG(e1.imdbRating) AS AVG_imdbRating 
		WHERE  AVG_imdbRating > 7.5 
	MATCH (e2:Person)
		WHERE e2.bornIn IN e0_bornIn
	WITH *
	RETURN  e2
	LIMIT 5000`

	var JSONQuery entity.IncomingQueryJSON
	json.Unmarshal(query, &JSONQuery)

	s := NewService()
	cypher, err := s.ConvertQuery(&JSONQuery)
	if err != nil {
		fmt.Println(err)
	}

	trimmedCypher := strings.Replace(*cypher, "\n", "", -1)
	trimmedCypher = strings.Replace(trimmedCypher, "\t", "", -1)

	trimmedAnswer := strings.Replace(answer, "\n", "", -1)
	trimmedAnswer = strings.Replace(trimmedAnswer, "\t", "", -1)

	fmt.Println(*cypher)
	assert.Equal(t, trimmedAnswer, trimmedAnswer)

}
func Test6(t *testing.T) {
	// Works, but, the AVG function is applied to a string, so that doesnt work, but the translation does :D
	query := []byte(`{
		"return": {
			"entities": [
				11,
				12
			],
			"relations": [
				10
			],
			"groupBys": []
		},
		"entities": [
			{
				"name": "Person",
				"ID": 11,
				"constraints": []
			},
			{
				"name": "Movie",
				"ID": 12,
				"constraints": []
			}
		],
		"relations": [
			{
				"ID": 10,
				"name": "DIRECTED",
				"depth": {
					"min": 1,
					"max": 1
				},
				"fromType": "entity",
				"fromID": 11,
				"toType": "entity",
				"toID": 12,
				"constraints": []
			}
		],
		"groupBys": [],
		"machineLearning": [],
		"limit": 5000,
		"databaseName": "Movies3"
	}
	`)

	var JSONQuery entity.IncomingQueryJSON
	json.Unmarshal(query, &JSONQuery)

	s := NewService()
	cypher, err := s.ConvertQuery(&JSONQuery)
	if err != nil {
		fmt.Println(err)
	}

	fmt.Println(*cypher)
	t.Fail()
}
